console.log("everything is fine\n"
           +"there are no errors\n"
           +"this output is not haiku.\n\n"
           +"is 8 ok?\n"
           +"ok, 8 can stay.\n"
           +"ok 100 might be confusing\n"
           +"  but: nevertheless, here we are\n"
           +"  this: is indented\n"
           +"  and: it\n"
           +"  might: ~\n"
           +"  be: yaml?\n"
           +"ok done now, exiting")
